# SPDX-FileCopyrightText: 2024-present ZanSara <github@zansara.dev>
# SPDX-License-Identifier: AGPL-3.0-or-later
"""
Init file for the intentional_textual_ui package.
"""
from intentional_textual_ui.bot_interface import TextualUIBotInterface

__all__ = ["TextualUIBotInterface"]
